#!/usr/bin/python

# Moore Observatory code to solve for dome azimuth given telescope pointing
# Copyright (c) 2010-2012 John Kielkopf, Jeff Hay and Karen Collins
# kielkopf@louisville.edu

# Date: April 2, 2012
# Version: 1.0

# This file is part of PyDome

# PyDome is free software: you can redistribute it and/or modify       
# it under the terms of the GNU General Public License as published by 
# the Free Software Foundation, either version 3 of the License, or    
# (at your option) any later version.                                  
#                                                                      
# PyDome is distributed in the hope that it will be useful,            
# but WITHOUT ANY WARRANTY; without even the implied warranty of       
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        
# GNU General Public License for more details.                         

from datetime import datetime

# Define global data for this site

# Longitude is + west of prime meridian 

# Longitude for Moore Observatory       
# sitelongitude = 85.5288888888

# Longitude for Mt. Kent Observatory    
# sitelongitude = -151.855528  

sitelongitude = 85.5288888888


# Map a time in hours to the range  0  to 24 */

def map24(hour):
  
  if (hour < 0.0): 
  
    n = int(hour / 24.0) - 1
    newhour = hour - n * 24.0
    return (newhour)
   
  elif (hour >= 24.0): 
  
    n = int(hour / 24.0)
    newhour= hour - n * 24.0
    return (newhour)
   
  else: 
    newhour = hour
    return (newhour)
  
# Fractional part

def frac(x):

  newx = x - int(x)
  if (newx < 0.):
    newx = newx + 1.
      
  return (newx)


# Calculate the current universal time in hours with millisecond resolution

def utnow():
  
  dt = datetime.utcnow()
  
  hours = dt.hour
  minutes = dt.minute
  seconds = dt.second
  microseconds = dt.microsecond
  ut = hours + minutes/60. + seconds/3600. + microseconds/3600000000.
  
  return (ut)
    
# Calculate the Julian day number for a given year, month, day, and universal time

def calcjd(year,month,day,ut):
  
  day = day + ut / 24.0;
  
  
  if int(month)== 1 or int(month)== 2:
    year = year - 1.
    month = month + 12.
  
  if (year + month/12.0 + day/365.25) >= (1582.0 + 10.0 / 12.0 + 15.0 / 365.25):
    na = int(year/100.)
    a = float(na)
    b = 2.0 - a + int(a/4.0)
  else:
    b = 0.
    
  if year < 0.:
    nc = int(float(int(365.25 * year)) - 0.75)
    c = float(nc)
  else:
    nc = int(365.25 * year)          
    c = float(nc)
  
  nd = int(30.6001 * (month + 1.)) 
  d = float(nd)
  jd = b + c + d + day + 1720994.5
  
  return (jd)

# Calculate the Julian day number at this moment

def jdnow():
  dt = datetime.utcnow()
  
  year = dt.year
  month = dt.month
  day = dt.day
  hours = dt.hour
  minutes = dt.minute
  seconds = dt.second
  microseconds = dt.microsecond
  ut = hours + minutes/60. + seconds/3600. + microseconds/3600000000.
  jd = calcjd(year, month, day, ut)
  
  return (jd)

# Calculate the sidereal time for a year, month, day and universal time at the define site latitude  
  
def calclst(year, month, day, ut, glong):
  jdeod =  calcjd(year, month, day, 0.0)
  tu = (jdeod - 2451545.0) / 36525.0
  t0 = (24110.54841 / 3600.0) + (8640184.812866 / 3600.0) * tu + (0.093104 / 3600.0) * tu * tu - (6.2e-6 / 3600.0) * tu * tu * tu
  t0 = map24(t0)
  gmst = map24(t0 + ut * 1.002737909)
  lmst = 24.0 * frac( (gmst - (glong / 15.0) ) / 24.0)
  
  return (lmst)
  
# Calculate the sidereal time at this moment for the defined site latitude

def lstnow():
  
  dt = datetime.utcnow()
  
  year = dt.year
  month = dt.month
  day = dt.day
  hours = dt.hour
  minutes = dt.minute
  seconds = dt.second
  microseconds = dt.microsecond
  ut = hours + minutes/60. + seconds/3600. + microseconds/3600000000.
  glong = sitelongitude
  lst = calclst(year,month,day,ut,glong)
  
  return (lst)

  
  

# Quick tests

print("UT now %f" % utnow())

print("JD now %f" % jdnow())

print("LST: %f" % lstnow())


 
