#!/usr/bin/python

# Moore Observatory RFID dome tag reader
# Copyright (c) 2010-2012 John Kielkopf and Jeff Hay
# kielkopf@louisville.edu

# Date:April 6, 2012
# Version: 1.0

# This file is part of PyDome

# PyDome is free software: you can redistribute it and/or modify       
# it under the terms of the GNU General Public License as published by 
# the Free Software Foundation, either version 3 of the License, or    
# (at your option) any later version.                                  
#                                                                      
# PyDome is distributed in the hope that it will be useful,            
# but WITHOUT ANY WARRANTY; without even the implied warranty of       
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        
# GNU General Public License for more details.                         

# Credit: Phidgets code downloaded on March 30, 2012
# Copyright: 2010 Phidgets Inc.
# License: Creative Commons Attribution 2.5 Canada License
# Author: Adam Stelmack
# Date: May 16, 2010

# Basic imports
from ctypes import *
import sys
import argparse
import time
import math

# Phidget specific imports
from Phidgets.PhidgetException import PhidgetErrorCodes, PhidgetException
from Phidgets.Events.Events import AttachEventArgs, DetachEventArgs, ErrorEventArgs, OutputChangeEventArgs, TagEventArgs
from Phidgets.Devices.RFID import RFID

parser= argparse.ArgumentParser(description='Monitor the RFID tag reader and find the azimuth angle')

if len(sys.argv) > 1:    
  sys.exit("Usage: monitor_dome")
  exit()   

# Import taglist and build a tag dictionary
tagfilename = "/usr/local/observatory/prefs/prefs.dome"
tagfile = open(tagfilename,'r')
if not tagfile:
  print("Taglist was not found in /usr/local/observatory/prefs")
  exit(1)

tag_dictionary = dict()

for newline in tagfile:

# Diagnostic if needed 
# print newline, 

  items = newline.split()  
  if len(items)==3:
    if items[0] == 'dome.tag=':
      tag = items[1]
      angle = float(items[2])
      tag_dictionary.update({tag:angle})
#     Alternatively form --  tag_dictionary[tag] = angle             

tagfile.close()
# print tag_dictionary
    

# Create an RFID object
try:
  rfid = RFID()

except RuntimeError as e:
  print("Runtime Exception: %s" % e.details)
  print("Exiting ... ")
  exit(1)

# Information Display Function
def displayDeviceInfo():
  print("|------------|----------------------------------|--------------|------------|")
  print("|- Attached -|-              Type              -|- Serial No. -|-  Version -|")
  print("|------------|----------------------------------|--------------|------------|")
  print("|- %8s -|- %30s -|- %10d -|- %8d -|" % (rfid.isAttached(), rfid.getDeviceName(), rfid.getSerialNum(), rfid.getDeviceVersion()))
  print("|------------|----------------------------------|--------------|------------|")
  print("Number of outputs: %i -- Antenna Status: %s -- Onboard LED Status: %s" % (rfid.getOutputCount(), rfid.getAntennaOn(), rfid.getLEDOn()))

# Event Handler Callback Functions
def rfidAttached(e):
  attached = e.device
  print("RFID %i Attached!" % (attached.getSerialNum()))

def rfidDetached(e):
  detached = e.device
  print("RFID %i Detached!" % (detached.getSerialNum()))

def rfidError(e):
  try:
    source = e.device
    print("RFID %i: Phidget Error %i: %s" % (source.getSerialNum(), e.eCode, e.description))
  except PhidgetException as e:
    print("Phidget Exception %i: %s" % (e.code, e.details))

def rfidOutputChanged(e):
  source = e.device
  print("RFID %i: Output %i State: %s" % (source.getSerialNum(), e.index, e.state))

def rfidTagGained(e):
  source = e.device
  rfid.setLEDOn(1)
  print("RFID %i: Tag Read: %s" % (source.getSerialNum(), e.tag))

def rfidTagLost(e):
  source = e.device
  rfid.setLEDOn(0)
  print("RFID %i: Tag Lost: %s" % (source.getSerialNum(), e.tag))

# Monitordome code begins here

# Initialize Phidgets tag reader
try:
  rfid.setOnAttachHandler(rfidAttached)
  rfid.setOnDetachHandler(rfidDetached)
  rfid.setOnErrorhandler(rfidError)
  rfid.setOnOutputChangeHandler(rfidOutputChanged)
  rfid.setOnTagHandler(rfidTagGained)
  rfid.setOnTagLostHandler(rfidTagLost)

except PhidgetException as e:
  print("Phidget Exception %i: %s" % (e.code, e.details))
  print("Exiting ...")
  exit(1)

# Connect to the tag reader
print("Connecting to the RFID reader ...")

try:
  rfid.openPhidget()
except PhidgetException as e:
  print("Phidget Exception %i: %s" % (e.code, e.details))
  print("Exiting ...")
  exit(1)

print("Waiting for tag reader ...")

try:
  rfid.waitForAttach(10000)

except PhidgetException as e:
  print("Phidget Exception %i: %s" % (e.code, e.details))
  try:
    rfid.closePhidget()
  except PhidgetException as e:
    print("Phidget Exception %i: %s" % (e.code, e.details))
    print("Exiting ...")
    exit(1)
  print("Exiting ...")
  exit(1)

else:
  displayDeviceInfo()


# Turn on the tag reader antenna 
print("Turning on the RFID antenna ...")
rfid.setAntennaOn(True)

# Wait for a new tag and monitor stdin for input to terminate
print("Dome monitor is on ...")

# Wait for any key
# chr = sys.stdin.read(1)

# print "Press Ctrl+z to exit ..."

while 1:
# Test for break condition would go here
  time.sleep(1.0)

# What was the last tag read and its angle in the dictionary?
  try:
    lasttag = rfid.getLastTag()
    tagstatus = open('/usr/local/observatory/status/dometag','w')
    tagstatus.write ("%s \n" % (lasttag))
    tagstatus.close()
    anglestatus = open('/usr/local/observatory/status/domeazimuth','w')
    angle = tag_dictionary[lasttag]
    anglestatus.write("%f \n" % angle)
    anglestatus.close()

  except PhidgetException as e:
    pass
#    print("Last Tag:  ")
#    print("Phidget Exception %i: %s" % (e.code, e.details))


# Close the tag reader connection
print("Closing dome monitor ...")

try:
  rfid.closePhidget()

except PhidgetException as e:
  print("Dome monitor Phidget exception %i: %s" % (e.code, e.details))
  print("Exiting with exception...")
  exit(1)

exit(0)
